use clap::{Parser, Subcommand};

use crate::{
    cli::{publish::PublishCommand, serve::ServeCommand, subscribe::SubscribeCommand},
    state::ClientState,
};

mod publish;
mod serve;
mod subscribe;

#[derive(Parser)]
#[command(author, version, about)]
struct Cli {
    #[command(subcommand)]
    subcommands: CliSubcommands,

    #[arg(
        long = "server-url",
        env = "NORUN_SERVER_URL",
        default_value = "http://localhost:4242"
    )]
    server_url: String,
}

#[derive(Subcommand)]
enum CliSubcommands {
    Subscribe(SubscribeCommand),
    Publish(PublishCommand),
    Serve(ServeCommand),
}

pub async fn execute() -> anyhow::Result<()> {
    let cmd = Cli::parse();

    let state = ClientState::new(&cmd.server_url);

    match cmd.subcommands {
        CliSubcommands::Publish(cmd) => cmd.execute(&state).await,
        CliSubcommands::Serve(cmd) => cmd.execute().await,
        CliSubcommands::Subscribe(cmd) => cmd.execute(&state).await,
    }
}
