use norun_grpc_interface::{registry_service_server::RegistryService, *};

use crate::{
    server::services::registry::{self, RegistryServiceState},
    state::ServerState,
};

pub struct GrpcRegistryService {
    pub state: ServerState,
}

#[async_trait::async_trait]
impl RegistryService for GrpcRegistryService {
    #[tracing::instrument(skip(self), level = "trace")]
    async fn publish(
        &self,
        request: tonic::Request<PublishRequest>,
    ) -> std::result::Result<tonic::Response<PublishResponse>, tonic::Status> {
        tracing::debug!("publish called");

        let req = request.into_inner();

        let project = req
            .project
            .ok_or(tonic::Status::invalid_argument("a project is required"))?;

        self.state
            .registry_service()
            .store(&project)
            .await
            .inspect_err(|e| tracing::warn!("failed to handle storage of registry item: {}", e))
            .map_err(|e| tonic::Status::internal(e.to_string()))?;

        Ok(tonic::Response::new(PublishResponse {}))
    }

    #[tracing::instrument(skip(self), level = "trace")]
    async fn get_topic(
        &self,
        request: tonic::Request<GetTopicRequest>,
    ) -> std::result::Result<tonic::Response<GetTopicResponse>, tonic::Status> {
        tracing::debug!("subscribe called");

        let req = request.into_inner();

        let projects = self
            .state
            .registry_service()
            .get_topic(&req.topic)
            .await
            .inspect_err(|e| tracing::warn!("failed to subscribe on topic: {}", e))
            .map_err(|e| tonic::Status::internal(e.to_string()))?;

        Ok(tonic::Response::new(GetTopicResponse {
            projects: Some(projects.into()),
        }))
    }
}

impl From<registry::Projects> for Projects {
    fn from(value: registry::Projects) -> Self {
        Self {
            projects: value.projects.into_iter().collect(),
        }
    }
}
